<?php

namespace App\Middleware;

use App\Util\AuthBasic;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

$app->add(function (Request $request, Response $response, $next) {
    $auth = $request->getHeaderLine('Authorization');
    $route = $request->getUri()->getPath();
    $route = preg_replace("/\/$/i", "", $route);
    $config = getClientName();

    $publicRoutes = $config && is_array($config['publicRoutes']) ? $config['publicRoutes'] : [];
    
    try {
        
       if(in_array($route, $publicRoutes) || in_array("/$route", $publicRoutes)) {
            $next($request, $response);
            return $response;
        }

        if (!$auth) {
            throw new \Exception('UNAUTHORIZED');
        }
        
        $credential = explode(' ', $auth, 2);
        if (count($credential) < 2 || strtolower($credential[0]) !== 'basic') {
            return $response->withStatus(400)->withJson(['error' => 'Invalid auth type']);
        }
    
        $decoded = base64_decode($credential[1]);
        
        if (!$decoded || !strpos($decoded, ':')) {
            return $response->withStatus(400)->withJson(['error' => 'Invalid credential encode']);
        }
    
        list($username, $password) = explode(':', $decoded, 2);

        if(!isset($config['authorizedUsers']) || !isset($config['authorizedUsers'][$username])) {
            throw new \Exception('UNAUTHORIZED');
        }
        
        $savedPass = $config['authorizedUsers'][$username];
        
        if (!AuthBasic::verifyPassword($config['encryptType'], $password, $savedPass)) {
            throw new \Exception('UNAUTHORIZED');
        }
    
        $next($request, $response);
        return $response;
    } catch (\Exception $e) {
        return $response->withStatus(401)->withHeader('WWW-Authenticate', 'Basic realm="Restricted Area"');
    }
    
});